#include "BinSolver.h"
#include "global.h"

#include "model.h"

#include<stdio.h>
#include<string.h>

// solver order: 
//		0:minisat
//		1: eureka
//		2:rsat
//		3: zchaff
// 		4: vallst
//		5:march
//		6:kcnfs
enum modelType {RM_MODEL=1, CM_MODEL=2, F_MODEL=0};

BinSolver* binSolvers[14];
BinSolver* LSSolver;
int NUM_ALGS = 14;

void BuildSolvers(char* strseed)
{

  // -- zchaff_rand
  binSolvers[3]=new BinSolver ("zchaff_rand", 3, 1);
 binSolvers[3]->argv[2] = strseed;
  binSolvers[3]->argv[3] = NULL;

  binSolvers[3]->models[F_MODEL] = new SATmodel(ZchaffModel, ZchaffFeatMean, ZchaffFeatStd, ZchaffFeatID1, ZchaffFeatID2, ZchaffNumFeat);
  binSolvers[3]->models[RM_MODEL] = new SATmodel(ZchaffRSModel, ZchaffRUModel, ZchaffRSFeatMean, ZchaffRSFeatStd, ZchaffRSFeatID1, ZchaffRSFeatID2, ZchaffClassCoeffRS, ZchaffClassCoeffRU, ZchaffRSNumFeat);
  binSolvers[3]->models[CM_MODEL] = new SATmodel(ZchaffCSModel, ZchaffCUModel, ZchaffCSFeatMean, ZchaffCSFeatStd, ZchaffCSFeatID1, ZchaffCSFeatID2, ZchaffClassCoeffCS, ZchaffClassCoeffCU, ZchaffCSNumFeat);

  // --vallst
  binSolvers[4]=new BinSolver ("vallst", 6, 1);
  binSolvers[4]->argv[2] = "-s";
  binSolvers[4]->argv[3] = strseed;
  binSolvers[4]->argv[4] = "-o";
  binSolvers[4]->argv[5] = "-";
  binSolvers[4]->argv[6] = NULL;

  binSolvers[4]->models[F_MODEL] = new SATmodel(VallstModel, VallstFeatMean, VallstFeatStd, VallstFeatID1, VallstFeatID2, VallstNumFeat);
  binSolvers[4]->models[RM_MODEL] = new SATmodel(VallstRSModel, VallstRUModel, VallstRSFeatMean, VallstRSFeatStd, VallstRSFeatID1, VallstRSFeatID2, VallstClassCoeffRS, VallstClassCoeffRU, VallstRSNumFeat);
  binSolvers[4]->models[CM_MODEL] = new SATmodel(VallstCSModel, VallstCUModel, VallstCSFeatMean, VallstCSFeatStd, VallstCSFeatID1, VallstCSFeatID2, VallstClassCoeffCS, VallstClassCoeffCU, VallstCSNumFeat);




  // -- march_dl 2004
  binSolvers[5]=new BinSolver ("march_dl", 2, 1);
  binSolvers[5]->argv[2] = NULL;

  binSolvers[5]->models[F_MODEL] = new SATmodel(MarchModel, MarchFeatMean, MarchFeatStd, MarchFeatID1, MarchFeatID2, MarchNumFeat);
  binSolvers[5]->models[RM_MODEL] = new SATmodel(MarchRSModel, MarchRUModel, MarchRSFeatMean, MarchRSFeatStd, MarchRSFeatID1, MarchRSFeatID2, MarchClassCoeffRS, MarchClassCoeffRU, MarchRSNumFeat);
  binSolvers[5]->models[CM_MODEL] = new SATmodel(MarchCSModel, MarchCUModel, MarchCSFeatMean, MarchCSFeatStd, MarchCSFeatID1, MarchCSFeatID2, MarchClassCoeffCS, MarchClassCoeffCU, MarchCSNumFeat);

  // -- minisat2.0
  binSolvers[0]=new BinSolver ("minisat20", 3, 1);
  binSolvers[0]->argv[2] = "foo";
  binSolvers[0]->argv[3] = NULL;

  binSolvers[0]->models[F_MODEL] = new SATmodel(MinisatModel, MinisatFeatMean, MinisatFeatStd, MinisatFeatID1, MinisatFeatID2, MinisatNumFeat);
  binSolvers[0]->models[RM_MODEL] = new SATmodel(MinisatRSModel, MinisatRUModel, MinisatRSFeatMean, MinisatRSFeatStd, MinisatRSFeatID1, MinisatRSFeatID2, MinisatClassCoeffRS, MinisatClassCoeffRU, MinisatRSNumFeat);
  binSolvers[0]->models[CM_MODEL] = new SATmodel(MinisatCSModel, MinisatCUModel, MinisatCSFeatMean, MinisatCSFeatStd, MinisatCSFeatID1, MinisatCSFeatID2, MinisatClassCoeffCS, MinisatClassCoeffCU, MinisatCSNumFeat);
  // -- Eureka
  binSolvers[1]=new BinSolver ("eureka", 2, 1);
  binSolvers[1]->argv[2] = NULL;

  binSolvers[1]->models[F_MODEL] = new SATmodel(EurekaModel, EurekaFeatMean, EurekaFeatStd, EurekaFeatID1, EurekaFeatID2, EurekaNumFeat);
  binSolvers[1]->models[RM_MODEL] = new SATmodel(EurekaRSModel, EurekaRUModel, EurekaRSFeatMean, EurekaRSFeatStd, EurekaRSFeatID1, EurekaRSFeatID2, EurekaClassCoeffRS, EurekaClassCoeffRU, EurekaRSNumFeat);
  binSolvers[1]->models[CM_MODEL] = new SATmodel(EurekaCSModel, EurekaCUModel, EurekaCSFeatMean, EurekaCSFeatStd, EurekaCSFeatID1, EurekaCSFeatID2, EurekaClassCoeffCS, EurekaClassCoeffCU, EurekaCSNumFeat);
  //-- rsat_rand
  binSolvers[2]=new BinSolver ("rsat", 3, 1);
  binSolvers[2]->argv[2] = "t";
  binSolvers[2]->argv[3] = NULL;

  binSolvers[2]->models[F_MODEL] = new SATmodel(RsatModel, RsatFeatMean, RsatFeatStd, RsatFeatID1, RsatFeatID2, RsatNumFeat);
  binSolvers[2]->models[RM_MODEL] = new SATmodel(RsatRSModel, RsatRUModel, RsatRSFeatMean, RsatRSFeatStd, RsatRSFeatID1, RsatRSFeatID2, RsatClassCoeffRS, RsatClassCoeffRU, RsatRSNumFeat);
  binSolvers[2]->models[CM_MODEL] = new SATmodel(RsatCSModel, RsatCUModel, RsatCSFeatMean, RsatCSFeatStd, RsatCSFeatID1, RsatCSFeatID2, RsatClassCoeffCS, RsatClassCoeffCU, RsatCSNumFeat);
  //-- kcnfs
  binSolvers[6]=new BinSolver ("kcnfs", 4, 1);
  binSolvers[6]->argv[2] = "-s";
  binSolvers[6]->argv[3] = strseed;
  binSolvers[6]->argv[4] = NULL;

  binSolvers[6]->models[F_MODEL] = new SATmodel(KcnfsModel, KcnfsFeatMean, KcnfsFeatStd, KcnfsFeatID1, KcnfsFeatID2, KcnfsNumFeat);
  binSolvers[6]->models[RM_MODEL] = new SATmodel(KcnfsRSModel, KcnfsRUModel, KcnfsRSFeatMean, KcnfsRSFeatStd, KcnfsRSFeatID1, KcnfsRSFeatID2, KcnfsClassCoeffRS, KcnfsClassCoeffRU, KcnfsRSNumFeat);
  binSolvers[6]->models[CM_MODEL] = new SATmodel(KcnfsCSModel, KcnfsCUModel, KcnfsCSFeatMean, KcnfsCSFeatStd, KcnfsCSFeatID1, KcnfsCSFeatID2, KcnfsClassCoeffCS, KcnfsClassCoeffCU, KcnfsCSNumFeat);

  // -- zchaff_rand WP
  binSolvers[10]=new BinSolver ("zchaff_rand", 3, 1);
 binSolvers[10]->argv[2] = strseed;
  binSolvers[10]->argv[3] = NULL;

  binSolvers[10]->models[RM_MODEL] = new SATmodel(ZchaffWPRSModel, ZchaffWPRUModel, ZchaffWPRSFeatMean, ZchaffWPRSFeatStd, ZchaffWPRSFeatID1, ZchaffWPRSFeatID2, ZchaffWPClassCoeffRS, ZchaffWPClassCoeffRU, ZchaffWPRSNumFeat);
  binSolvers[10]->models[CM_MODEL] = new SATmodel(ZchaffWPCSModel, ZchaffWPCUModel, ZchaffWPCSFeatMean, ZchaffWPCSFeatStd, ZchaffWPCSFeatID1, ZchaffWPCSFeatID2, ZchaffWPClassCoeffCS, ZchaffWPClassCoeffCU, ZchaffWPCSNumFeat);

  // --vallst WP
  binSolvers[11]=new BinSolver ("vallst", 6, 1);
  binSolvers[11]->argv[2] = "-s";
  binSolvers[11]->argv[3] = strseed;
  binSolvers[11]->argv[4] = "-o";
  binSolvers[11]->argv[5] = "-";
  binSolvers[11]->argv[6] = NULL;

  binSolvers[11]->models[RM_MODEL] = new SATmodel(VallstWPRSModel, VallstWPRUModel, VallstWPRSFeatMean, VallstWPRSFeatStd, VallstWPRSFeatID1, VallstWPRSFeatID2, VallstWPClassCoeffRS, VallstWPClassCoeffRU, VallstWPRSNumFeat);
  binSolvers[11]->models[CM_MODEL] = new SATmodel(VallstWPCSModel, VallstWPCUModel, VallstWPCSFeatMean, VallstWPCSFeatStd, VallstWPCSFeatID1, VallstWPCSFeatID2, VallstWPClassCoeffCS, VallstWPClassCoeffCU, VallstWPCSNumFeat);




  // -- march_dl 2004 WP
  binSolvers[12]=new BinSolver ("march_dl", 2, 1);
  binSolvers[12]->argv[2] = NULL;

  binSolvers[12]->models[RM_MODEL] = new SATmodel(MarchWPRSModel, MarchWPRUModel, MarchWPRSFeatMean, MarchWPRSFeatStd, MarchWPRSFeatID1, MarchWPRSFeatID2, MarchWPClassCoeffRS, MarchWPClassCoeffRU, MarchWPRSNumFeat);
  binSolvers[12]->models[CM_MODEL] = new SATmodel(MarchWPCSModel, MarchWPCUModel, MarchWPCSFeatMean, MarchWPCSFeatStd, MarchWPCSFeatID1, MarchWPCSFeatID2, MarchWPClassCoeffCS, MarchWPClassCoeffCU, MarchWPCSNumFeat);

  // -- minisat2.0
  binSolvers[7]=new BinSolver ("minisat20", 3, 1);
  binSolvers[7]->argv[2] = "foo";
  binSolvers[7]->argv[3] = NULL;

  binSolvers[7]->models[RM_MODEL] = new SATmodel(MinisatWPRSModel, MinisatWPRUModel, MinisatWPRSFeatMean, MinisatWPRSFeatStd, MinisatWPRSFeatID1, MinisatWPRSFeatID2, MinisatWPClassCoeffRS, MinisatWPClassCoeffRU, MinisatWPRSNumFeat);
  binSolvers[7]->models[CM_MODEL] = new SATmodel(MinisatWPCSModel, MinisatWPCUModel, MinisatWPCSFeatMean, MinisatWPCSFeatStd, MinisatWPCSFeatID1, MinisatWPCSFeatID2, MinisatWPClassCoeffCS, MinisatWPClassCoeffCU, MinisatWPCSNumFeat);
  // -- Eureka
  binSolvers[8]=new BinSolver ("eureka", 2, 1);
  binSolvers[8]->argv[2] = NULL;

  binSolvers[8]->models[RM_MODEL] = new SATmodel(EurekaWPRSModel, EurekaWPRUModel, EurekaWPRSFeatMean, EurekaWPRSFeatStd, EurekaWPRSFeatID1, EurekaWPRSFeatID2, EurekaWPClassCoeffRS, EurekaWPClassCoeffRU, EurekaWPRSNumFeat);
  binSolvers[8]->models[CM_MODEL] = new SATmodel(EurekaWPCSModel, EurekaWPCUModel, EurekaWPCSFeatMean, EurekaWPCSFeatStd, EurekaWPCSFeatID1, EurekaWPCSFeatID2, EurekaWPClassCoeffCS, EurekaWPClassCoeffCU, EurekaWPCSNumFeat);
  //-- rsat_rand
  binSolvers[9]=new BinSolver ("rsat", 3, 1);
  binSolvers[9]->argv[2] = "t";
  binSolvers[9]->argv[3] = NULL;

  binSolvers[9]->models[RM_MODEL] = new SATmodel(RsatWPRSModel, RsatWPRUModel, RsatWPRSFeatMean, RsatWPRSFeatStd, RsatWPRSFeatID1, RsatWPRSFeatID2, RsatWPClassCoeffRS, RsatWPClassCoeffRU, RsatWPRSNumFeat);
  binSolvers[9]->models[CM_MODEL] = new SATmodel(RsatWPCSModel, RsatWPCUModel, RsatWPCSFeatMean, RsatWPCSFeatStd, RsatWPCSFeatID1, RsatWPCSFeatID2, RsatWPClassCoeffCS, RsatWPClassCoeffCU, RsatWPCSNumFeat);
  //-- kcnfs
  binSolvers[13]=new BinSolver ("kcnfs", 4, 1);
  binSolvers[13]->argv[2] = "-s";
  binSolvers[13]->argv[3] = strseed;
  binSolvers[13]->argv[4] = NULL;

  binSolvers[13]->models[RM_MODEL] = new SATmodel(KcnfsWPRSModel, KcnfsWPRUModel, KcnfsWPRSFeatMean, KcnfsWPRSFeatStd, KcnfsWPRSFeatID1, KcnfsWPRSFeatID2, KcnfsWPClassCoeffRS, KcnfsWPClassCoeffRU, KcnfsWPRSNumFeat);
  binSolvers[13]->models[CM_MODEL] = new SATmodel(KcnfsWPCSModel, KcnfsWPCUModel, KcnfsWPCSFeatMean, KcnfsWPCSFeatStd, KcnfsWPCSFeatID1, KcnfsWPCSFeatID2, KcnfsWPClassCoeffCS, KcnfsWPClassCoeffCU, KcnfsWPCSNumFeat);


  LSSolver=new BinSolver ("ubcsat2006", 17, 2);
  LSSolver->argv[1] = "-inst";
  LSSolver->argv[3] = "-alg";
  LSSolver->argv[4] = "saps";
  LSSolver->argv[5] = "-alpha";
  LSSolver->argv[6] = "1.3";
  LSSolver->argv[7] = "-rho";
  LSSolver->argv[8] = "0.1";
  LSSolver->argv[9] = "-cutoff";
  LSSolver->argv[10] = "max";
  LSSolver->argv[11] = "-timeout";
  LSSolver->argv[12] = LS_PREP_TIMEOUT;
//  LSSolver->argv[13] = "-solve";
  LSSolver->argv[13] = "-seed";
  LSSolver->argv[14] = strseed;
  LSSolver->argv[15] = "-r";
  LSSolver->argv[16] = "satcomp";
  LSSolver->argv[17] = NULL;  
 

}
int runLocalSearch(char* inputFile, char* strseed)
{
  char* argv[] = {"ubcsat",
                  "-alg", "saps",  "-i", inputFile, "-runs", "1",
                  "-alpha", "1.3", "-rho", "0.1",  "-timeout", LS_PREP_TIMEOUT,
                  "-seed", strseed,
                  "-solve"};
  //		  "-r", "satcomp", vlineFilename,

  int argc = 16;

  printf("c satzilla is running local search for 30s\n");
  return  ubcsat::main(argc, argv);

}
